"""
RVS - A Git-like Version Control System
Implements Git-like functionality without requiring Git installation.
"""
__version__ = "1.1.0"
__author__ = "RVS Team"
__description__ = "A Git-like version control system implemented in Python"
from .core.repository import RVS
from .exceptions import RVSError, RepositoryError, ObjectError
__all__ = [
    "RVS",
    "RVSError", 
    "RepositoryError",
    "ObjectError",
    "__version__"
]
